/*
  Calisto - An internet Talker
  Copyright (C) 1998-2001 Peter Howkins, Matthew Howkins and Simon Howkins

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,     
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License   
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Email: calisto@calisto.cx

  $Id: privs.c,v 1.9 2001/03/08 16:07:09 peter Exp $

 */
static const char rcsid[] = "$Id: privs.c,v 1.9 2001/03/08 16:07:09 peter Exp $";

#include "library.h"
#include "privs.h"
#include "strplus.h"
#include "structs.h"

/* Ranks */
#define RANK_GUEST    0
#define RANK_RESIDENT 1
#define RANK_SU       2
#define RANK_ADMIN    3
#define RANK_HA       4

#define RANK_MAX      4

/* Preferences */
#define PREF_NONE     0

#define PREF_COLOUR   (1 << 0)
#define PREF_NUTS     (1 << 1)

/* Privilidges */
#define PRIV_NONE          0 /* used to blank someones privs */
/* basic guest and ressie privs */
#define PRIV_BASE          (1 << 0)
#define PRIV_TALK          (1 << 1)
#define PRIV_SAVE          (1 << 2)
#define PRIV_BS1           (1 << 3)
#define PRIV_BS2           (1 << 4)
#define PRIV_BS3           (1 << 5)
#define PRIV_BS4           (1 << 6)
#define PRIV_BS5           (1 << 7)
#define PRIV_BS6           (1 << 8)
#define PRIV_BS7           (1 << 9)

/* su privs */
#define PRIV_SU_BASE       (1 << 10)
#define PRIV_SU_BUMP       (1 << 11)
#define PRIV_SU_TRACE      (1 << 12)
#define PRIV_SUS2          (1 << 13)
#define PRIV_SUS3          (1 << 14)
#define PRIV_SUS4          (1 << 15)
#define PRIV_SUS5          (1 << 16)
#define PRIV_SUS6          (1 << 17)
#define PRIV_SUS7          (1 << 18)
#define PRIV_SUS8          (1 << 19)

/* admin privs */
#define PRIV_AD_BASE       (1 << 20)
#define PRIV_AD_SHUTDOWN   (1 << 21)
#define PRIV_AD_GRANT      (1 << 22)
#define PRIV_ADS2          (1 << 23)
#define PRIV_ADS3          (1 << 24)
#define PRIV_ADS4          (1 << 25)
#define PRIV_ADS5          (1 << 26)
#define PRIV_ADS6          (1 << 27)
#define PRIV_ADS7          (1 << 28)
#define PRIV_ADS8          (1 << 29)

/* spare privs */
#define PRIV_S30           (1 << 30)
#define PRIV_S31           (1 << 31)

/* combination privs */
#define C_PRIV_GUEST    (PRIV_TALK | PRIV_BASE)
#define C_PRIV_RESIDENT (C_PRIV_GUEST | PRIV_SAVE)
#define C_PRIV_SU       (C_PRIV_RESIDENT | PRIV_SU_TRACE | PRIV_SU_BUMP | \
                         PRIV_SU_BASE)
#define C_PRIV_ADMIN    (C_PRIV_SU | PRIV_AD_BASE)
#define C_PRIV_HA       (C_PRIV_ADMIN | PRIV_AD_SHUTDOWN | PRIV_AD_GRANT)


typedef struct {
  const char *rankname;
  const int rank;
  const char *cprivname;
} rank;

typedef struct {
  const char *prefname;
  const unsigned long pref;
} pref;

typedef struct {
  const char *privname;
  const unsigned long priv;
  const char *col;
} priv;

static const rank ranks[] = {
  { "guest",    RANK_GUEST    , "guest" },
  { "resident", RANK_RESIDENT , "resident" },
  { "su",       RANK_SU       , "su" },
  { "admin",    RANK_ADMIN    , "admin" },
  { "ha",       RANK_HA       , "ha" },
};
static const unsigned rankn = sizeof ranks / sizeof(rank);

static const pref prefs[] = {
  { "colour",    PREF_COLOUR },
  { "color",     PREF_COLOUR },
  { "nuts",      PREF_NUTS },
};
static const unsigned prefn = sizeof prefs / sizeof(pref);

static const priv privs[] = {
   /* basic guest and ressie privs */
  { "n_base",     PRIV_BASE,       "^n" },
  { "n_talk",     PRIV_TALK,       "^n" },
  { "n_save",     PRIV_SAVE,       "^n" },
   /* su privs */
  { "s_base",     PRIV_SU_BASE,    "^c" },
  { "s_trace",    PRIV_SU_TRACE,   "^c" },
  { "s_bump",     PRIV_SU_BUMP,    "^c" },
   /* admin privs */
  { "a_base",     PRIV_AD_BASE,    "^r" },
  { "a_shutdown", PRIV_AD_SHUTDOWN,"^P" },
  { "a_grant",    PRIV_AD_GRANT,   "^P" }
};
static const unsigned privn = sizeof privs / sizeof(priv);

static const priv combo_privs[] = {
   /* combination privs */
  { "guest",      C_PRIV_GUEST,    "^n" },
  { "resident",   C_PRIV_RESIDENT, "^n" },
  { "su",         C_PRIV_SU,       "^c" },
  { "admin",      C_PRIV_ADMIN,    "^r" },
  { "ha",         C_PRIV_HA,       "^P" }
};
static const unsigned combo_privn = sizeof combo_privs / sizeof(priv);

#if 0
/* there is no use for this function yet */
/* take a rank number and return a rank pointer, or null */
static const rank *ranknum_to_rank(unsigned ranknum)
{
  unsigned i;
 
  for(i=0; i<rankn; i++) {
    if(ranks[i].rank == ranknum)
      return &ranks[i];
  }
  return NULL;
}
#endif

/* take a rank name and return a rank pointer, or null */
static const rank *rankname_to_rank(const char *rankname)
{
  unsigned i;
 
  for(i=0; i<rankn; i++) {
    if(STREQ(ranks[i].rankname, rankname))
      return &ranks[i];
  }
  return NULL;
}

bool rankname_exists(const char *rankname)
{
  return (rankname_to_rank(rankname) != NULL);
}

/* Takes a pref name, and returns a pref pointer, or NULL if not found */
static const pref *prefname_to_pref(const char *prefname)
{
  unsigned i;

  for (i = 0; i < prefn; i++) {
    if (STRIEQ(prefname, prefs[i].prefname))
      return &prefs[i];
  }

  return NULL;
}

/* Takes a priv name, and returns a priv pointer, or NULL if not found */
static const priv *privname_to_priv(const char *privname)
{
  unsigned i;

  /* combo privs */
  for (i = 0; i < privn; i++) {
    if (STRIEQ(privname, privs[i].privname))
      return &privs[i];
  }
  /* combo privs */
  for (i = 0; i < combo_privn; i++) {
    if (STRIEQ(privname, combo_privs[i].privname))
      return &combo_privs[i];
  }

  return NULL;
}

bool privname_exists(const char *privname)
{
  return (privname_to_priv(privname) != NULL);
}

bool prefname_exists(const char *prefname)
{
  return (prefname_to_pref(prefname) != NULL);
}

bool haspref(const character *c, const char *prefname)
{
  const pref *p = prefname_to_pref(prefname);

  if (p) {
    return (c->prefs & p->pref);
  } else {
    return FALSE;
  }
}

bool haspriv(const character *c, const char *privname)
{
  const priv *p = privname_to_priv(privname);

  if (p) {
    return (c->privs & p->priv);
  } else {
    return FALSE;
  }
}

void setpriv(character *c, const char *privname)
{
  const priv *p = privname_to_priv(privname);

  if (p)
    c->privs |= p->priv;
}

void setpref(character *c, const char *prefname)
{
  const pref *p = prefname_to_pref(prefname);

  if (p)
    c->prefs |= p->pref;
}

void clearpriv(character *c, const char *privname)
{
  const priv *p = privname_to_priv(privname);

  if (p)
    c->privs &= ~p->priv;
}

void clearpref(character *c, const char *prefname)
{
  const pref *p = prefname_to_pref(prefname);

  if (p)
    c->prefs &= ~p->pref;
}

const char *privcol(const char *privname)
{
  const priv *p = privname_to_priv(privname);

  if (p) {
    return p->col;
  } else {
    return NULL;
  }
}

void privs_list_all(character *c)
{
  unsigned i;

  send_to_char(c, "Available privs:\n");  
  for (i = 0; i < privn; i++)
    send_to_char(c, "%s%s^n ", privs[i].col, privs[i].privname);
  send_to_char(c, "\nCombo privs:\n");  
  for (i = 0; i < combo_privn; i++)
    send_to_char(c, "%s%s^n ", combo_privs[i].col, combo_privs[i].privname);
  send_to_char(c, "\n");
}

void privs_list_player(character *c, character *target)
{
  unsigned i;

  for (i = 0; i < privn; i++)
    if (target->privs & privs[i].priv)
      send_to_char(c, " %s%s", privs[i].col, privs[i].privname);
  send_to_char(c, "^n %08lx", target->privs);
}

void setrank(character *c, const char *rankname)
{
  const rank *r = rankname_to_rank(rankname);

  if(!r)
    return;

  if(r->rank == c->rank)
    return;

  if(r->rank < c->rank) {
    /* a demotion */
    c->privs = PRIV_NONE;
    setpriv(c, r->cprivname);
  }

  if(r->rank > c->rank) {
    /* a promotion */
    setpriv(c, r->cprivname);
  }

  c->rank = r->rank;
}
