/*
  Calisto - An internet Talker
  Copyright (C) 1998-2001 Peter Howkins, Matthew Howkins and Simon Howkins

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,     
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License   
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Email: calisto@calisto.cx

  $Id: strplus.c,v 1.6 2001/03/07 21:08:47 peter Exp $

 */
static const char rcsid[] = "$Id: strplus.c,v 1.6 2001/03/07 21:08:47 peter Exp $";

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "strplus.h"

/* portable strdup */
char *STRDUP(const char *s)
{
  char *new;
  new = malloc(strlen(s) +1); /* +1 for terminator */
  if(new == NULL)
    return NULL;
  strcpy(new, s);
  return new;
}

void strmunch(char *source, char **first, char **rest)
{
  char *term;
  char *f, *r;

  /* find initial non white space char in source */
  while(*source && (isspace(*source)))
    source++;
  f = source;
  r  = source;
  /* move 'rest' to the first white-space char after 'first */
  while (*r && (!isspace(*r)))
    r ++;
  /* make a note of where the terminator will go */
  term = r;
  /* Now move 'rest' on to next non white-space char */
  while (*r && (isspace(*r)))
    r ++;
  /* insert the terminator after the first word */
  *term = '\0';
  
  *first = f;
  *rest  = r;
}

int strchomp(char *s)
{
  if (s) {
    int len = strlen(s);

    if (s[len - 1] == '\n') {
      s[len - 1] = '\0';
      return 1;
    }
  }
  return 0;
}

char *strlower(char *s)
{
  char *beg = s;

  while (*s) {
    *s = tolower(*s);
    s++;
  }
  return beg;
}

char *strupper(char *s)
{
  char *beg = s;
  while (*s) {
    *s = toupper(*s);
    s++;
  }
  return beg;
}

int stricmp(const char *s1, const char *s2)
{
  char i, j;

  do {
    i = *s1++;
    j = *s2++;

    i = tolower(i);
    j = tolower(j);
  } while (i && i == j);

  return i - j;
}

int strnicmp(const char *s1, const char *s2, size_t n)
{
  char i, j;

  if (!n)
    return 0;
 
  do {
    i = *s1++;
    j = *s2++;
    i = tolower(i);
    j = tolower(j);
  } while (i && i==j && --n);

  return i - j;
}


/*
  String versions of ctype functions
*/

bool strisalnum(const char *s)
{
  while (*s) {
    if (!isalnum(*s))
      return FALSE;
    s++;
  }
  return TRUE;
}

bool strisalpha(const char *s)
{
  while (*s) {
    if (!isalpha(*s))
      return FALSE;
    s++;
  }
  return TRUE;
}

bool striscntrl(const char *s)
{
  while (*s) {
    if (!iscntrl(*s))
      return FALSE;
    s++;
  }
  return TRUE;
}

bool strisdigit(const char *s)
{
  while (*s) {
    if (!isdigit(*s))
      return FALSE;
    s++;
  }
  return TRUE;
}

bool strisgraph(const char *s)
{
  while (*s) {
    if (!isgraph(*s))
      return FALSE;
    s++;
  }
  return TRUE;
}

bool strislower(const char *s)
{
  while (*s) {
    if (!islower(*s))
      return FALSE;
    s++;
  }
  return TRUE;
}

bool strisprint(const char *s)
{
  while (*s) {
    if (!isprint(*s))
      return FALSE;
    s++;
  }
  return TRUE;
}

bool strispunct(const char *s)
{
  while (*s) {
    if (!ispunct(*s))
      return FALSE;
    s++;
  }
  return TRUE;
}

bool strisspace(const char *s)
{
  while (*s) {
    if (!isspace(*s))
      return FALSE;
    s++;
  }
  return TRUE;
}

bool strisupper(const char *s)
{
  while (*s) {
    if (!isupper(*s))
      return FALSE;
    s++;
  }
  return TRUE;
}

bool strisxdigit(const char *s)
{
  while (*s) {
    if (!isxdigit(*s))
      return FALSE;
    s++;
  }
  return TRUE;
}

char *strip_leading_white_space(const char *s)
{
  while (isspace(*s))
    s++;
  return (char *) s;
}

void strip_trailing_white_space(char *s)
{
  char *pos;
  int len = strlen(s); 

  if (len > 0) {
    pos = s + len - 1; /* pointing to last char in string proper (not \0) */
    while(pos >= s && isspace(*pos)) {
      pos--;
    }
    *(pos+1) = '\0';
  }
}

